# YAHPO Gym - Train
[![Module Handbook](https://img.shields.io/badge/Website-Documentation-blue)](https://slds-lmu.github.io/yahpo_gym/) 
[![Paper](https://img.shields.io/badge/arXiv-Paper-blue)](https://arxiv.org/abs/2109.03670)
[![Software (Python)](https://img.shields.io/badge/Software-Python-green)](https://github.com/slds-lmu/yahpo_gym/tree/main/yahpo_gym)
[![Software (R)](https://img.shields.io/badge/Software-R-green)](https://github.com/slds-lmu/yahpo_gym/tree/main/yahpo_gym_r)

Train and evaluate surrogates for HPO problems.

Code for tuning and training the surrogates can be found in the `notebooks` folder https://github.com/slds-lmu/yahpo_gym/blob/main/yahpo_train/notebooks/tune_resnet.py
