# Credits


## Development

- Carolin Benjamins (Lead)
- Helena Graf
- Sarah Segel
- Difan Deng
- Tim Ruhkopf
- Leona Hennig
- Soham Basu
- Edward Bergman

## Contributors

None yet. Why not be the first?
