# Using Carps

In the case when you are developing your optimizer or benchmark in a standalone package, you can use carps without
directly working in the carps repo.

## With your own benchmark
For a custom benchmark we have an [example repo](https://github.com/automl/OptBench).
It shows how to use your own benchmark with carps optimizers.

## With your own optimizer
When you develop your own optimizer, check this [example repo](https://github.com/automl/CARP-S-example-optimizer).
It shows how to use your own optimizer with carps benchmarks.