% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{list_benchmarks}
\alias{list_benchmarks}
\title{Print available benchmarks}
\usage{
list_benchmarks()
}
\value{
A [`Configuration`] (Python Object)
}
\description{
Print available benchmarks
}
